﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MvcPagingGrid.Models;
using MvcPagingGrid.Models.ViewData;

namespace MvcPagingGrid.Controllers
{
    [HandleError]
    public class HomeController : Controller
    {
        private CarModel _carModel;

        public HomeController() : this(new CarModel()) { }

        public HomeController(CarModel carModel)
        {
            _carModel = carModel;
        }

        public ActionResult Index()
        {
            CarViewData viewData = new CarViewData();
            string sortExpr = !string.IsNullOrEmpty(GetString("SortExpr")) ? GetString("SortExpr") : "Make";
            string sortDir = !string.IsNullOrEmpty(GetString("SortDir")) ? GetString("SortDir") : "ASC";
            int pageNumber = GetInt("Page");

            viewData.Cars = _carModel.GetCars(sortExpr, sortDir, pageNumber, 10);
            viewData.CarCount = _carModel.GetCarsCount();

            return View("Index", viewData);
        }

        public ActionResult View(int id)
        {
            Car car = _carModel.GetCarById(id);
            return View("View", car);
        }

        public ActionResult About()
        {
            return View();
        }

        private string GetString(string s)
        {
            return HttpUtility.UrlDecode(this.ControllerContext.HttpContext.Request.QueryString[s]);
        }

        private int GetInt(string s)
        {
            int id;
            if (int.TryParse(HttpUtility.UrlDecode(this.ControllerContext.HttpContext.Request.QueryString[s]), out id))
            {
                return id;
            }

            // if an error, return 0
            return 0;
        }
    }
}
