﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Collections.Specialized;
using System.Web.Mvc;

namespace MvcPagingGrid.Helpers
{
    public static class RedirectExtensions
    {
        public const string RedirectKey = "RedirectUrl";

        public static string ReadRedirectHyperLink(this HtmlHelper htmlHelper)
        {
            return ReadRedirectHyperLink(htmlHelper, new RedirectProperties());
        }

        public static string ReadRedirectHyperLink(this HtmlHelper htmlHelper, RedirectProperties redirectProperties)
        {
            string url = ReadFromRedirectUrl(redirectProperties);

            TagBuilder tag = new TagBuilder("a")
            {
                InnerHtml = redirectProperties.Text
            };
            tag.MergeAttributes(new Dictionary<string, string> { { "href", url }, { "onclick", redirectProperties.OnClick }, { "class", redirectProperties.Class } });

            return tag.ToString();
        }

        public static string AddRedirectHyperLink(this HtmlHelper htmlHelper)
        {
            return AddRedirectHyperLink(htmlHelper, new RedirectProperties());
        }

        public static string AddRedirectHyperLink(this HtmlHelper htmlHelper, RedirectProperties redirectProperties)
        {
            string url = GenerateRedirectUrl(redirectProperties);

            TagBuilder tag = new TagBuilder("a")
            {
                InnerHtml = redirectProperties.Text
            };
            tag.MergeAttributes(new Dictionary<string, string> { { "href", url }, { "class", redirectProperties.Class } });

            return tag.ToString();
        }

        public static string CanMaintainRedirectHyperLink(this HtmlHelper htmlHelper)
        {
            return CanMaintainRedirectHyperLink(htmlHelper, new RedirectProperties());
        }

        public static string CanMaintainRedirectHyperLink(this HtmlHelper htmlHelper, RedirectProperties redirectProperties)
        {
            //set up the redirect
            string redirectUrl = QueryString.GetString(RedirectKey);
            if (string.IsNullOrEmpty(redirectUrl) && !string.IsNullOrEmpty(redirectProperties.BackupHref))
                redirectUrl = redirectProperties.BackupHref;

            //now add to the requested href
            if (!string.IsNullOrEmpty(redirectUrl))
            {
                if (redirectProperties.Href.Contains("?"))
                    redirectProperties.Href += "&";
                else
                    redirectProperties.Href += "?";

                redirectProperties.Href += string.Format("{0}={1}", RedirectKey, HttpUtility.UrlEncode(redirectUrl));
            }

            TagBuilder tag = new TagBuilder("a")
            {
                InnerHtml = redirectProperties.Text
            };
            tag.MergeAttributes(new Dictionary<string, string> { { "href", redirectProperties.Href }, { "class", redirectProperties.Class }, { "onclick", redirectProperties.OnClick } });


            return tag.ToString();
        }

        public static string GenerateRedirectUrl(RedirectProperties properties)
        {
            string redirectUrl = HttpContext.Current.Request.Path;

            string qString = QueryString.SetValues(new NameValueCollection(), HttpContext.Current.Request.RawUrl);

            if (qString.Length > 0)
                redirectUrl += "?" + qString;


            if (!properties.Href.Contains("?"))
                properties.Href += "?";
            else
                properties.Href += "&";

            properties.Href += string.Format("{0}={1}", RedirectKey, HttpUtility.UrlEncode(redirectUrl));

            return properties.Href;
        }

        public static string ReadFromRedirectUrl(RedirectProperties properties)
        {
            string url = string.Empty;
            string redirectURL = QueryString.GetString(RedirectKey);
            if (!string.IsNullOrEmpty(redirectURL))
            {
                if (redirectURL.Contains("?"))
                {
                    int i = redirectURL.IndexOf("?") + 1;
                    NameValueCollection collection = QueryString.CreateRedirectNameValueCollection(HttpContext.Current.Request.QueryString[RedirectKey]);
                    string qs = QueryString.FormatQueryString(collection);
                    redirectURL = redirectURL.Substring(0, i) + qs;
                }

                url = redirectURL;
            }
            if (string.IsNullOrEmpty(url))
            {
                if (!string.IsNullOrEmpty(properties.BackupHref))
                    url = properties.BackupHref;
            }

            return url;
        }
    }

    public class RedirectProperties
    {
        public string Text { get; set; }
        public string Href { get; set; }
        public string OnClick { get; set; }
        public string BackupHref { get; set; }
        public string Class { get; set; }
    }
}
