﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Runtime.Serialization;
using System.IO;
using System.Xml.Serialization;
using System.Xml;

namespace MvcPagingGrid.Models
{
    [Serializable]
    public class Car : IComparable<Car>
    {
        public int Id { get; set; } 
        public string Make { get; set; }
        public string Model { get; set; }
        public int Year { get; set; }
        public string Color { get; set; }
        public int Miles { get; set; }
        public int Doors { get; set; }
        public bool Sunroof { get; set; }

        public Car() { }

        #region IComparable<Car> Members
        public int CompareTo(Car other)
        {
            return string.Compare(this.Make, other.Make);
        }
        #endregion
    }

    public class CarSearchComparer : IComparer<Car>
    {
        #region IComparer<Car> Members
        private string sort;
        public CarSearchComparer(string sort)
        {
            this.sort = sort;
        }

        public int Compare(Car x, Car y)
        {
            int compareValue = 0;

            switch (this.sort)
            {
                case "Make":
                    compareValue = string.Compare(x.Make, y.Make);
                    break;
                case "Model":
                    compareValue = string.Compare(x.Model, y.Model);
                    break;
                case "Year":
                    if (x.Year > y.Year)
                        compareValue = 1;
                    else if (x.Year < y.Year)
                        compareValue = -1;
                    
                    break;
                case "Color":
                    compareValue = string.Compare(x.Color, y.Color);
                    break;
                case "Miles":
                    if (x.Miles > y.Miles)
                        compareValue = 1;
                    else if (x.Miles < y.Miles)
                        compareValue = -1;

                    break;
                case "Doors":
                    if (x.Doors > y.Doors)
                        compareValue = 1;
                    else if (x.Doors < y.Doors)
                        compareValue = -1;

                    break;
            }
            return compareValue;
        }

        #endregion
    }



    public class CarModel
    {
        public List<Car> GetCars(string sortExpression, string sortDirection, int page, int pageSize)
        {
            List<Car> cars = new List<Car>();
            Stream stream = null;
            try
            {
                stream = File.Open(HttpContext.Current.Server.MapPath("~/Content/data.xml"), FileMode.Open);
                XmlSerializer x = new XmlSerializer(typeof(List<Car>));

                XmlTextReader reader = new XmlTextReader(stream);
                cars = (List<Car>)x.Deserialize(reader);
                stream.Close();
                reader.Close();
                cars.Sort(new CarSearchComparer(sortExpression));

                if (sortDirection.Contains("DESC"))
                    cars.Reverse();

                if (pageSize > 0)
                {
                    int startIndex = page * pageSize;
                    int itemCount = pageSize;
                    startIndex = startIndex > 0 ? startIndex - 1 : startIndex;
                    if (startIndex + itemCount > cars.Count)
                        itemCount = cars.Count - startIndex;
                    cars = cars.GetRange(startIndex, itemCount);
                }
                return cars;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public Car GetCarById(int carId)
        {
            Car car = new Car();

            List<Car> cars = GetCars("Make", "ASC", 0, 0);

            car = cars.Find(x => x.Id == carId);

            return car;
        }

        public int GetCarsCount()
        {
            List<Car> cars = new List<Car>();
            Stream stream = null;
            try
            {
                stream = File.Open(HttpContext.Current.Server.MapPath("~/Content/data.xml"), FileMode.Open);
                XmlSerializer x = new XmlSerializer(typeof(List<Car>));

                XmlTextReader reader = new XmlTextReader(stream);
                cars = (List<Car>)x.Deserialize(reader);
                stream.Close();
                reader.Close();
                return cars.Count;
            }
            catch (Exception)
            {
                throw;
            }
        }
    }
}
